;; * Emacs Q Script

(message "ENTERING emacs-ESU-script ...")

;; **  Coding system

(message "emacs-ESU-script coding system ...")

(setq locale-coding-system 'utf-8)
(set-terminal-coding-system 'utf-8)
(set-keyboard-coding-system 'utf-8)
(set-selection-coding-system 'utf-8)
(prefer-coding-system 'utf-8)

;; ** Screen Colors

;; window type dependend screen colors
(defun tj/set-window-type-dependend-emacsclient-colors ()
    "Sets emacsclient colors for X.
Background-color is set to black and foreground-color to wheat for
emacsclient on X"
    (if (display-graphic-p)
        (progn
          (set-background-color "black")
          (set-foreground-color "wheat"))))

(add-hook 'server-visit-hook
          'tj/set-window-type-dependend-emacsclient-colors)

(add-hook 'emacs-startup-hook
          'tj/set-window-type-dependend-emacsclient-colors)


;; ** Helper functions

(message "emacs-ESU-script helper functions ...")

;; *** Improved require

(defvar missing-packages-list nil
  "List of packages that `try-require' can't find.")

;; attempt to load a feature/library, failing silently
(defun try-require (feature)
  "Attempt to load a library or module. Return true if the
library given as argument is successfully loaded. If not, instead
of an error, just add the package to a list of missing packages."
  (condition-case err
      ;; protected form
      (progn
        (message "Checking for library `%s'..." feature)
        (if (stringp feature)
            (load-library feature)
          (require feature))
        (message "Checking for library `%s'... Found" feature))
    ;; error handler
    (file-error  ; condition
     (progn
       (message "Checking for library `%s'... Missing" feature)
       (add-to-list 'missing-packages-list feature 'append))
     nil)))


;; *** Fast temporary buffers

(defun tj/switch-to-tmp-buffer ()
  "Opens a temporary buffer"
  (interactive)
  (switch-to-buffer (generate-new-buffer-name "tmp")))

;; define shortcut for switching to temp-buffer
(global-set-key (kbd "C-x t") 'tj/switch-to-tmp-buffer)

;; **  General settings

(message "emacs-ESU-script general settings ...")

;; To disable the menubar, use the following line:
(menu-bar-mode -1)
;; To disable the scrollbar, use the following line:
(scroll-bar-mode -1)
;; To disable the toolbar, use the following line:
(tool-bar-mode -1)
;; syntax highlighting
(global-font-lock-mode t)
;; sane select (mark) mode
;; (transient-mark-mode t)
;; entry deletes marked text
(delete-selection-mode t)
;; match parentheses
(show-paren-mode t)
;; wrap long lines in text mode
(add-hook 'text-mode-hook 'turn-on-auto-fill)

;; enable "dired on steroids"
(ido-mode t)
;; ;; Enable versioning with default values.
;; (setq version-control t)
;; Save all backup file into the designated directory.
(setq backup-directory-alist (quote ( (".*" . "/home/tj/.emacs.d/backups/"))))
;; move between windows (shift-key + arrows)
(windmove-default-keybindings)
;; "y or n" instead of "yes or no"
(fset 'yes-or-no-p 'y-or-n-p)

;; **  Warning about being 'root'

(message "emacs-ESU-script root warning ...")

(defface find-file-root-header-face
  '((t (:foreground "white" :background "red3")))
  "*Face use to display header-lines for files opened as root.")

(defun find-file-root-header-warning ()
  "*Display a warning in header line of the current buffer.
  This function is suitable to add to `find-file-root-hook'."
  (let* ((warning "WARNING: EDITING FILE WITH ROOT PRIVILEGES!")
	 (space (+ 6 (- (frame-width) (length warning))))
	 (bracket (make-string (/ space 2) ?-))
	 (warning (concat bracket warning bracket)))
    (setq header-line-format
	  (propertize warning 'face 'find-file-root-header-face))))

(add-hook 'find-file-hook 'find-file-root-header-warning)
(add-hook 'dired-mode-hook 'find-file-root-header-warning)

;; ** Load libraries

(message "emacs-ESU-script load libraries ...")

;; *** Smex

(add-to-list 'load-path "/home/tj/.emacs.d/elpa/smex-2.0")

(when (try-require 'smex)
(smex-initialize)

(global-set-key (kbd "M-x") 'smex)
(global-set-key (kbd "C-x x") 'smex)
(global-set-key (kbd "M-X") 'smex-major-mode-commands))

;; *** Switch windos

(add-to-list 'load-path "/home/tj/.emacs.d/elpa/switch-window-0.9/")

(try-require 'switch-window)

;; *** Window numbering

(add-to-list 'load-path "/home/tj/.emacs.d/src/")

(when (try-require 'window-numbering)
  (window-numbering-mode 1))

;; *** Winner mode

;; (add-to-list 'load-path "/usr/share/emacs/24.2/lisp/")

(when (try-require 'winner)
   (when (fboundp 'winner-mode)
   (winner-mode 1))

   (global-set-key (kbd "<f5>") 'winner-undo)
   (global-set-key (kbd "<f6>") 'winner-redo))


;; *** Missing packages

;; warn that some packages were missing
(if missing-packages-list
    (progn (message "Packages not found: %S" missing-packages-list)))


;; **  Visit dired buffers

(message "emacs-ESU-script visit dired buffer ...")

(dired "/usr/share/")
(split-window-sensibly (get-buffer-window (current-buffer)))
(dired "/usr/share/emacs/24.3/lisp/")

(message "Loading emacs-ESU-script (almost) DONE")


;; ** Custom Set Variables/Fonts

(message "FINALLY Custom vars/fonts ...")

(custom-set-faces
 ;; custom-set-faces was added by Custom.
 ;; If you edit it by hand, you could mess it up, so be careful.
 ;; Your init file should contain only one such instance.
 ;; If there is more than one, they won't work right.
 '(default ((t (:family "DejaVu Sans Mono" :foundry "unknown" :slant normal :weight normal :height 120 :width normal))))
 '(org-mode-line-clock ((t (:background "grey75" :foreground "red" :box (:line-width -1 :style released-button)))) t))
